//
// Copyright (C) 2023 Autodesk, Inc.
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of version 2.1 of the GNU Lesser
// General Public License as published by the Free Software Foundation.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

#include "stdafx.h"
#include "FdoWfsGetClassNamesCommand.h"

/*  Do not implement the copy constructor to avoid a lot of problems.
FdoWfsGetClassNamesCommand::FdoWfsGetClassNamesCommand(const FdoWfsGetClassNamesCommand &right)
{
}
*/

FdoWfsGetClassNamesCommand::FdoWfsGetClassNamesCommand(FdoIConnection *connection)
    : FdoCommonCommand<FdoIGetClassNames, FdoWfsConnection>(connection)
{
}

FdoWfsGetClassNamesCommand::~FdoWfsGetClassNamesCommand()
{
}

FdoString* FdoWfsGetClassNamesCommand::GetSchemaName()
{
    return mSchemaName;
}

void FdoWfsGetClassNamesCommand::SetSchemaName(FdoString* value)
{
    mSchemaName = value;
}

FdoStringCollection* FdoWfsGetClassNamesCommand::Execute()
{
    if (nullptr == mConnection)
        throw FdoException::Create(L"Connection not established");

    FdoPtr<FdoFeatureSchemaCollection> schemas = mConnection->GetDummySchemas();
    if (nullptr == schemas)
        throw FdoException::Create(L"Schemas not loaded");

    FdoPtr<FdoStringCollection> classNames = FdoStringCollection::Create();

    for (int i = 0; i < schemas->GetCount(); i++)
    {
        FdoPtr<FdoFeatureSchema> schema = schemas->GetItem(i);
        FdoPtr<FdoClassCollection> classes = schema->GetClasses();

        if (mSchemaName != L"")
        {
            if (mSchemaName == schema->GetName())
            {
                for (int j = 0; j < classes->GetCount(); j++)
                {
                    FdoPtr<FdoClassDefinition> classDef = classes->GetItem(j);
                    classNames->Add(classDef->GetQualifiedName());
                }
                break;
            }
        }
        else
        {
            for (int j = 0; j < classes->GetCount(); j++)
            {
                FdoPtr<FdoClassDefinition> classDef = classes->GetItem(j);
                classNames->Add(classDef->GetQualifiedName());
            }
        }
    }

    return classNames.Detach();
}

