/*
 * Copyright (C) 2004-2006  Autodesk, Inc.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of version 2.1 of the GNU Lesser
 * General Public License as published by the Free Software Foundation.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include "stdafx.h"
#include "FdoWfsGetFeature.h"
#include <OWS/FdoOwsOgcFilterSerializer.h>
#include "FdoWfsGlobals.h"
// uncomment this line to get a limited number of records
// this line have been added for debug purposes
//#define DEBUG_LIMIT_FEATURES

FdoWfsGetFeature::FdoWfsGetFeature(FdoString* targetNamespace,
                                    FdoString* targetNamespaceName,
                                    FdoString* srsName,
                                    FdoStringCollection* propertiesToSelect,
                                    FdoString* from,
                                    FdoFilter* where,
                                    FdoString* schemaName,
                                    FdoString* version,
                                    FdoBoolean invertAxis,
                                    FdoBoolean restrictToBBox) : FdoOwsRequest(FdoWfsGlobals::WFS, FdoWfsGlobals::GetFeature),
                                    m_targetNamespace(targetNamespace), m_targetNamespaceName(targetNamespaceName), m_srsName(srsName),
                                    m_propertiesToSelect(propertiesToSelect),
                                    m_from(from), m_where(where), m_schemaName(schemaName), m_invertAxis(invertAxis), m_restrictToBBox(restrictToBBox)
{
    m_encodeWithClassName = false;
    m_encodeWithNamespaceName = false;
	FdoOwsRequest::SetVersion (version ? version : FdoWfsGlobals::WfsVersion);
    FDO_SAFE_ADDREF(propertiesToSelect);
    FDO_SAFE_ADDREF(where);
}

FdoWfsGetFeature::~FdoWfsGetFeature()
{
}

FdoWfsGetFeature* FdoWfsGetFeature::Create(FdoString* targetNamespace, FdoString* targetNamespaceName, FdoString* srsName, FdoStringCollection* propertiesToSelect,
                            FdoString* from, FdoFilter* where, FdoString* schemaName, FdoString* version, FdoBoolean invertAxis, FdoBoolean restrictToBBox)
{
    return new FdoWfsGetFeature(targetNamespace, targetNamespaceName, srsName, propertiesToSelect, from, where, schemaName, version, invertAxis, restrictToBBox);
}

FdoStringP FdoWfsGetFeature::EncodeKVP()
{
    // for common request, version and service
    FdoStringP ret = FdoOwsRequest::EncodeKVP();
#ifdef DEBUG_LIMIT_FEATURES
    if (m_version == FdoWfsGlobals::WfsVersion200)
    {
        ret += L"&COUNT=50";
    }
    else
    {
        ret += FdoWfsGlobals::And;
        ret += L"maxFeatures";
        ret += FdoWfsGlobals::Equal;
        ret += L"50";
    }
#endif
    // TYPENAME, mandatory
    ret += FdoWfsGlobals::And;
    if (m_version == FdoWfsGlobals::WfsVersion200)
        ret += FdoWfsGlobals::TYPENAMES;
    else
        ret += FdoWfsGlobals::TYPENAME;
    ret += FdoWfsGlobals::Equal;
    if (m_schemaName.GetLength() != 0)
        ret += UrlEscape(m_schemaName + L":" + m_from);
    else
        ret += UrlEscape(m_from);
    // PROPERTYNAME, optional
    FdoInt32 numProps = 0;
    if (m_propertiesToSelect != NULL)
        numProps = m_propertiesToSelect->GetCount();
    if (numProps > 0)
    {
        ret += FdoWfsGlobals::And;
        ret += FdoWfsGlobals::PROPERTYNAME;
        ret += FdoWfsGlobals::Equal;
        FdoStringP prop = UrlEscape(m_propertiesToSelect->GetString(0));
        if (m_encodeWithClassName && !prop.Contains(L"/"))
        {
            ret += UrlEscape(m_from); // can we skip escaping this?
            ret += L"/";
        }
        ret += prop;
        for (int i = 1; i < numProps; i++)
        {
            prop = UrlEscape(m_propertiesToSelect->GetString(i));
            ret += FdoWfsGlobals::Comma;
            if (m_encodeWithClassName && !prop.Contains(L"/"))
            {
                ret += UrlEscape(m_from); // can we skip escaping this?
                ret += L"/";
            }
            ret += prop;
        }
    }
    // FILTER, optional
    if (m_where != NULL)
    {
        FdoPtr<FdoIoMemoryStream> stream = FdoIoMemoryStream::Create();
        FdoPtr<FdoXmlWriter> writer = FdoXmlWriter::Create(stream, false);
        writer->WriteStartElement(FdoWfsGlobals::Filter);

        // make "ogc" default namespace
        writer->WriteAttribute(FdoXml::mXmlnsPref, FdoWfsGlobals::OgcUri);
        // gml namespace
        FdoStringP ns = FdoXml::mXmlnsPref;
        ns += L":";
        ns += FdoXml::mGmlNs;
        writer->WriteAttribute(ns, FdoXml::mGmlUri);
        // application namespace
        if (m_targetNamespace != L"" && m_targetNamespaceName != L"")
        {
            ns = FdoXml::mXmlnsPref;
            ns += L":";
            ns += m_targetNamespaceName;
            writer->WriteAttribute(ns, m_targetNamespace);
        }

        FdoStringP prefix;
        if (m_encodeWithNamespaceName && m_targetNamespaceName != L"")
            prefix = m_targetNamespaceName;
        FdoOwsOgcFilterSerializer::Serialize(m_where, writer, m_srsName, prefix, m_invertAxis, m_restrictToBBox);

        writer = NULL;
        stream->Reset();
        FdoInt64 length = stream->GetLength();
        char* buffer = new char[(size_t)length + 1];
        stream->Read((FdoByte*)buffer, (FdoSize)length);
        buffer[length] = '\0';
        // we must get rid of the preceding  <?xml version="1.0" encoding="UTF-8" ?>
        char* buffer1 = buffer;
        while (*buffer1 != '>') buffer1++;
        buffer1++;
        FdoStringP filter = buffer1;
        delete[] buffer;

        // Some servers don't support BBOX filter, but support BBOX keyword only.
        // BBOX keyword and Filter keyword are mutually exclusive in some servers. If find BBOX filter, discard other parts of the filter.
        if (filter.Contains(FdoWfsGlobals::BBOX))
        {
            // From: <BBOX><PropertyName>wkb_geometry</PropertyName><gml:Envelope srsName="urn:ogc:def:crs:EPSG::4326"><gml:lowerCorner>44.609935 -0.626517</gml:lowerCorner><gml:upperCorner>47.538490 3.070080</gml:upperCorner></gml:Envelope></BBOX>
            // To: BBOX=44.609935,-0.626517,47.538490,3.070080,urn:ogc:def:crs:EPSG::4326
            FdoStringP filter1 = filter.Left(FdoWfsGlobals::TagBBoxStart);
            FdoStringP bboxFilter = filter.Right(FdoWfsGlobals::TagBBoxStart);
            FdoStringP filter2 = bboxFilter.Right(FdoWfsGlobals::TagBBoxEnd);
            filter = filter1 + filter2;

            bboxFilter = bboxFilter.Left(FdoWfsGlobals::TagBBoxEnd);
            FdoStringP lowerCorner = bboxFilter.Right(FdoWfsGlobals::TagLowerCornerStart);
            lowerCorner = lowerCorner.Left(FdoWfsGlobals::TagLowerCornerEnd);
            lowerCorner = lowerCorner.Replace(L" ", FdoWfsGlobals::Comma);
            FdoStringP upperCorner = bboxFilter.Right(FdoWfsGlobals::TagUpperCornerStart);
            upperCorner = upperCorner.Left(FdoWfsGlobals::TagUpperCornerEnd);
            upperCorner = upperCorner.Replace(L" ", FdoWfsGlobals::Comma);

            FdoStringP bbox = lowerCorner + FdoWfsGlobals::Comma + upperCorner;
            if (m_srsName != L"" && !m_srsName.Contains(L"/"))
            {
                bbox += FdoWfsGlobals::Comma;
                bbox += m_srsName;
            }
            ret += FdoWfsGlobals::And;
            ret += FdoWfsGlobals::BBOX;
            ret += FdoWfsGlobals::Equal;
            ret += UrlEscape(bbox);

            // keyword SRSNAME is mandatory for some servers
            if (m_srsName != L"" && !m_srsName.Contains(L"/"))
            {
                ret += FdoWfsGlobals::And;
                ret += FdoWfsGlobals::SRSNAME;
                ret += FdoWfsGlobals::Equal;
                ret += m_srsName;
            }
        }
        else
        {
            ret += FdoWfsGlobals::And;
            ret += FdoWfsGlobals::Filter;
            ret += FdoWfsGlobals::Equal;
            ret += UrlEscape(filter);
        }
    }
    if (ret.Contains(FdoWfsGlobals::Dot))
        ret = ret.Replace(FdoWfsGlobals::Dot, L".");
    return ret;

}

FdoStringP FdoWfsGetFeature::EncodeXml()
{
    return L"";
}







