//
// Copyright (C) 2023 Autodesk, Inc.
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of version 2.1 of the GNU Lesser
// General Public License as published by the Free Software Foundation.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


#include "stdafx.h"
#include "FdoWfsGetSchemaNamesCommand.h"

/*  Do not implement the copy constructor to avoid a lot of problems.
FdoWfsGetSchemaNamesCommand::FdoWfsGetSchemaNamesCommand(const FdoWfsGetSchemaNamesCommand &right)
{
}
*/

FdoWfsGetSchemaNamesCommand::FdoWfsGetSchemaNamesCommand(FdoIConnection *connection)
    : FdoCommonCommand<FdoIGetSchemaNames, FdoWfsConnection>(connection)
{
}

FdoWfsGetSchemaNamesCommand::~FdoWfsGetSchemaNamesCommand()
{
}

FdoStringCollection* FdoWfsGetSchemaNamesCommand::Execute()
{
    if (nullptr == mConnection)
        throw FdoException::Create(L"Connection not established");

    FdoPtr<FdoFeatureSchemaCollection> schemas = mConnection->GetDummySchemas();
    if (nullptr == schemas)
        throw FdoException::Create(L"Schemas not loaded");

    FdoPtr<FdoStringCollection> schemaNames = FdoStringCollection::Create();
    for (int i = 0; i < schemas->GetCount(); i++)
    {
        FdoPtr<FdoFeatureSchema> schema = schemas->GetItem(i);
        FdoStringP schemaName = schema->GetQualifiedName();
        schemaNames->Add(schemaName);
    }

    return schemaNames.Detach();
}

